import * as fs from "fs";

const path = "./cache/getGitActivity";
const validTime = 1000 * 60 * 30; // 30 min in milliseconds

export type GitActivityCache = {
  data: Map<string, number>,
  lastRefresh: number
}

export default (req, res) => {
  if(fs.existsSync(path)) {
    try {
      let data: GitActivityCache = JSON.parse(fs.readFileSync(path).toString());

      if(Date.now() - data.lastRefresh < validTime) {
        res.statusCode = 200;
        res.send(JSON.stringify(data));
      }
      else {
        getData()
          .then((data) => {
            res.statusCode = 200;
            res.send(data)
          })
          .catch(() => {
            res.statusCode = 200;
            res.send(data.data);
          });
      }
    } catch (e) {
      getData()
        .then((data) => {
          res.statusCode = 500;
          res.send(data)
        })
        .catch(() => {
          res.statusCode = 500;
          res.send("Error while fetching data.");
        });
    }
  }
  else {
    getData()
      .then((data) => {
        res.statusCode = 200;
        res.send(data)
      })
      .catch(() => {
        res.statusCode = 500;
        res.send("Error while fetching data.");
      });
  }
}


async function getData(): Promise<string> {
  let gitData = await getGitData();

  fs.writeFileSync(path, JSON.stringify({data: gitData, lastRefresh: Date.now()}));

  return JSON.stringify(gitData);
}


async function getGitData(): Promise<Array<{ date: string, count: number}>> {
  console.log("Refreshing Data");

  let token = "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";
  let username = "networkException";

  const output: { [date: string]: number } = {};

  const headers = {
    'Authorization': `bearer ${token}`,
  };

  const body = {
    'query': `query {
        user(login: "${username}") {
            name
            contributionsCollection {
                contributionCalendar {
                    totalContributions
                    weeks {
                        contributionDays {
                            contributionCount
                            date
                        }
                    }
                }
            }
        }
    }`
  };

  const githubResponse = await fetch('https://api.github.com/graphql', { method: 'POST', body: JSON.stringify(body), headers });

  (await githubResponse.json()).data.user.contributionsCollection.contributionCalendar.weeks.forEach((week: any) => {
    week.contributionDays.forEach((day: any) => {
      if (day.date in output) {
        output[day.date] = output[day.date] + Number(day.contributionCount);
      }
      else {
        output[day.date] = Number(day.contributionCount);
      }
    });
  });

  const gitlabResponse: { [date: string]: number } = await (await fetch('https://gitlab.upi.li/users/networkException/calendar.json')).json();

  for (const date in gitlabResponse) {
    if (date in output) {
      output[date] = output[date] + gitlabResponse[date];
    }
    else {
      output[date] = gitlabResponse[date];
    }
  }

  return Object.keys(output).map(date => { return { date, count: output[date] }; });
}